import 'dart:async';
import 'package:uuid/uuid.dart';
import 'package:flutter/services.dart';
import 'package:flutter/foundation.dart';

const MethodChannel _sshChannel = const MethodChannel('ssh');

enum TypeOfAuthentication {
  key,
  password,
}

class AuthenticationMethod {
  TypeOfAuthentication typeOfAuthentication;
  String password;
  Map<String, String> key;

  AuthenticationMethod.password({
    @required String password,
  }) {
    this.typeOfAuthentication = TypeOfAuthentication.password;
  }

  AuthenticationMethod.key({
    @required Map<String, String> key,
  }) {
    this.typeOfAuthentication = TypeOfAuthentication.key;
  }

  dynamic get authentication {
    if (this.typeOfAuthentication == TypeOfAuthentication.password) {
      return this.password;
    } else if (this.typeOfAuthentication == TypeOfAuthentication.key) {
      return this.key;
    }
  }
}

class Endpoint {
  String host;
  int    port;

  Endpoint({
    @required this.host,
    @required this.port,
  });
}

class SSHClientConfiguration {
  String               user;
  AuthenticationMethod authenticationMethod;

  SSHClientConfiguration({
    @required this.user,
    @required this.authenticationMethod,
  });
}

class SSHTunnel {
  String                 id;
  Endpoint               tunnel;
  Endpoint               server;
  Endpoint               remote;
  SSHClientConfiguration clientConfiguration;

  String get tunnelBindIp   => this.tunnel.host;
  int    get tunnelBindPort => this.tunnel.port;

  SSHTunnel({
    @required this.tunnel,
    @required this.server,
    @required this.remote,
    @required this.clientConfiguration,
  }) {
    var uuid = new Uuid();
    this.id  = uuid.v4();
  }

  Future<String> start() async {
    Map<String, dynamic> params = new Map<String,String>();
    params["id"]         = this.id;
    params["tunnelHost"] = this.tunnel.host.toString();
    params["tunnelPort"] = this.tunnel.port;
    params["serverHost"] = this.server.host.toString();
    params["serverPort"] = this.server.port;
    params["remoteHost"] = this.server.host.toString();
    params["remotePort"] = this.server.port;
    params["user"]       = this.clientConfiguration.user;

    params["typeOfAuthentication"] = this.clientConfiguration.authenticationMethod.typeOfAuthentication;
    params["authentication"]       = this.clientConfiguration.authenticationMethod.authentication;

    var result = await _sshChannel.invokeMethod('start', params);
    return result;
  }

  Future<String> stop() async {
    var result = await _sshChannel.invokeMethod('stop', {});
    return result;
  }
}
